﻿Public Class MainForm

    Private Const AnimatedRectangleSize As Integer = 48
    Private Const FixedRectangleSize As Integer = 32
    Private Shared ReadOnly RNG As New Random()

    ' The animated rectangle moves around the screen. The fixed rectangle will always
    ' be in the center of the panel. The velocity controls how much the animated
    ' rectangle moves each frame.
    Private _animatedRectangle As New Rectangle
    Private _fixedRectangle As New Rectangle
    Private _velocity As Point

    ' Manually checks collisions using logic discussed in the article. Updates a label
    ' with collision data.
    Private Sub CheckCollisionsManually()
        Dim r1 As Rectangle = _fixedRectangle
        Dim r2 As Rectangle = _animatedRectangle

        ' Check collisions swapping order of r1 and r2 to catch the case where r2 contains r1.
        ' This is bad and I need to fix it.
        Dim isHorizontalCollision As Boolean = CheckHorizontalCollisionsManually(r1, r2)
        Dim isVerticalCollision As Boolean = CheckVerticalCollisionsManually(r1, r2)
        Dim isContainsCollision As Boolean = CheckContainsCollisionManually(r1, r2)

        lblManualTest.Text = String.Format("H: {0}, V: {1}, C: {2}", isHorizontalCollision, isVerticalCollision, isContainsCollision)
    End Sub

    Private Function CheckContainsCollisionManually(ByVal r1 As Rectangle, ByVal r2 As Rectangle) As Boolean
        Dim isContainsCollision As Boolean = False
        If r2.Left < r1.Left AndAlso r1.Left < r2.Right Then
            ' r1's left edge is inside r2. Check if its top edge is as well.
            If r2.Top < r1.Top AndAlso r1.Top < r2.Bottom Then
                ' The top and left edges of r1 are in r2; it must be inside of r2.
                isContainsCollision = True
            End If
        End If

        Return isContainsCollision
    End Function

    Private Function CheckHorizontalCollisionsManually(ByVal r1 As Rectangle, ByVal r2 As Rectangle) As Boolean
        Dim isHorizontalCollision As Boolean = False
        ' Left edge of r2
        If r1.Left < r2.Left AndAlso r2.Left < r1.Right Then
            isHorizontalCollision = True
        End If

        ' Right edge of r2
        If r1.Left < r2.Right AndAlso r2.Right < r1.Right Then
            isHorizontalCollision = True
        End If

        Return isHorizontalCollision
    End Function

    Private Function CheckVerticalCollisionsManually(ByVal r1 As Rectangle, ByVal r2 As Rectangle) As Boolean
        Dim isVerticalCollision As Boolean = False
        ' Top edge of r2
        If r1.Top < r2.Top AndAlso r2.Top < r1.Bottom Then
            isVerticalCollision = True
        End If

        ' Bottom edge of r2
        If r1.Top < r2.Bottom AndAlso r2.Bottom < r1.Bottom Then
            isVerticalCollision = True
        End If

        Return isVerticalCollision
    End Function


    ' Picks a new velocity for the animated rectangle if it's hit one of the walls.
    Private Sub DoAnimatedRectangleLogic()
        ' If the rectangle hit the left wall, give it a random X velocity that
        ' moves it to the right.
        If _animatedRectangle.Left <= 0 Then
            _velocity.X = RNG.Next(1, 11)
        End If

        ' If the rectangle hit the right wall, give it a random X velocity that
        ' moves it to the left.
        If _animatedRectangle.Right >= pnlPlayfield.Width Then
            _velocity.X = RNG.Next(-10, 0)
        End If

        ' If the rectangle this the bottom wall, give it a random Y velocity
        ' that moves it upward.
        If _animatedRectangle.Bottom >= pnlPlayfield.Height Then
            _velocity.Y = RNG.Next(-10, 0)
        End If

        ' If the rectangle hits the top wall, give it a random Y velocity
        ' that moves it downward
        If _animatedRectangle.Top <= 0 Then
            _velocity.Y = RNG.Next(1, 11)
        End If
    End Sub

    ' Uses Rectangle.IntersectsWith() to detect collisions and updates a label.
    Private Sub CheckCollisionsWithIntersect()
        Dim r1 As Rectangle = _fixedRectangle
        Dim r2 As Rectangle = _animatedRectangle

        Dim isCollision As Boolean = r1.IntersectsWith(r2)
        lblIntersectsTest.Text = String.Format("Collision: {0}", isCollision)
    End Sub

    ' Moves the animated rectangle
    Private Sub MoveAnimatedRectangle()
        Dim x As Integer = _animatedRectangle.X + _velocity.X
        Dim y As Integer = _animatedRectangle.Y + _velocity.Y

        _animatedRectangle = New Rectangle(x, y, AnimatedRectangleSize, AnimatedRectangleSize)
    End Sub

    ' The fixed rectangle displays in the center of the panel.
    Private Sub PlaceFixedRectangle()
        Dim x As Integer = (pnlPlayfield.Width - FixedRectangleSize) \ 2
        Dim y As Integer = (pnlPlayfield.Height - FixedRectangleSize) \ 2

        _fixedRectangle = New Rectangle(x, y, FixedRectangleSize, FixedRectangleSize)
    End Sub

    ' Initialize the location of the fixed rectangle and the animated rectangle when the
    ' form displays.
    Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
        MyBase.OnLoad(e)

        PlaceFixedRectangle()
        _velocity = New Point(5, 5)
        tmrAnimation.Start()
        Invalidate()
    End Sub

    ' Moves the animated rectangle, updates its velocity, and checks for collisions each frame.
    Private Sub tmrAnimation_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles tmrAnimation.Tick
        MoveAnimatedRectangle()
        DoAnimatedRectangleLogic()

        CheckCollisionsManually()
        CheckCollisionsWithIntersect()

        pnlPlayfield.Invalidate()
    End Sub

    ' Draw each rectangle when the playfield is invalidated
    Private Sub pnlPlayfield_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles pnlPlayfield.Paint
        e.Graphics.DrawRectangle(Pens.CornflowerBlue, _fixedRectangle)
        e.Graphics.DrawRectangle(Pens.Crimson, _animatedRectangle)
    End Sub

    ' When the playfield is resized, move the fixed rectangle
    Private Sub pnlPlayfield_SizeChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles pnlPlayfield.SizeChanged
        PlaceFixedRectangle()
        pnlPlayfield.Invalidate()
    End Sub
End Class
